import { Module, MutationTree, ActionTree, GetterTree } from 'vuex';
import { StateInterface } from 'src/store';
import { axios } from 'boot/axios';
import { LoginData } from 'src/plugins/user/models';
import { AxiosResponse } from 'axios';
import { LocalStorage, Loading } from 'quasar';
import { Router } from 'src/router';

export interface UserStateInterface extends LoginResponse {
  loginLoading: boolean;
}

export interface LoginResponse {
  user: FG.User;
  session: FG.Session;
}

const empty_session: FG.Session = {
          browser: '',
          expires: new Date(),
          lifetime: -1,
          platform: '',
          token: ''
};

const empty_user: FG.User = {
    display_name: '',
    firstname: '',
    lastname: '',
    mail: '',
    roles: [],
    userid: ''
};

const state: UserStateInterface = {
  user: empty_user,
  session: empty_session,
  loginLoading: false
};

const mutations: MutationTree<UserStateInterface> = {
  setUser(state, data: FG.User) {
    state.user = data;
  },
  setSession(state, data: FG.Session) {
    state.session = data;
  },
  setLoginLoading(state, data: boolean) {
    state.loginLoading = data;
  },
  showState(state) {
    console.log(state);
  }
};

const actions: ActionTree<UserStateInterface, StateInterface> = {
  login({ commit }, data: LoginData) {
    commit('setLoginLoading', true);
    Loading.setDefaults({
      spinner: () => import('src/components/loading/DarkCircularProgress.vue')
    });
    Loading.show({
      message: 'Du wirst eingeloggt'
    });
    void axios
      .post('/auth', data)
      .then((response: AxiosResponse<LoginResponse>) => {
        response.data.session.expires = new Date(response.data.session.expires);
        commit('setUser', response.data.user);
        commit('setSession', response.data.session);
        commit('showState');
        LocalStorage.set('user', response.data.user);
        LocalStorage.set('session', response.data.session);

        void Router.push({ name: 'user' });
      })
      .catch(error => {
        console.exception(error);
      })
      .finally(() => {
        commit('setLoginLoading', false);
        Loading.hide();
      });
  },

  doLogout({commit}, token: string) {
    Loading.show({ message: 'Du wirst ausgeloggt' });
    void axios
      .delete(`/auth/${token}`)
      .then(() => {
        commit('setUser', empty_user);
        commit('setSession', empty_session);
      })
      .finally(() => {
        LocalStorage.remove('user');
        LocalStorage.remove('session');
        Loading.hide();
      });
  },

  logout({ dispatch}, token: string) {
    dispatch('doLogout', token).finally(() => {void Router.push({ name: 'login' });});
  },

  updateUser({ commit, state }, data) {
    commit('setLoginLoading', true);
    axios
      .put(`/users/${state.user.userid}`, data)
      .catch(error => {
        console.log(error);
      })
      .finally(() => {
        commit('setLoginLoading', false);
      });
  },

  loadFromLocalStorage({ commit }) {
    console.log('load from store');
    let data = LocalStorage.getItem('user');
    commit(
      'setUser',
      data
        ? data
        : empty_user
    );
    data = LocalStorage.getItem('session');
    commit(
      'setSession',
      data
        ? data
        : empty_session
    );
    commit('showState');
  }
};

const getters: GetterTree<UserStateInterface, StateInterface> = {
  user({ user }) {
    return user;
  },
  displayName({ user }) {
    return user.display_name;
  },
  session({ session }) {
    return session;
  },
  loginLoading({ loginLoading }) {
    return loginLoading;
  }
};

const userStore: Module<UserStateInterface, StateInterface> = {
  namespaced: true,
  actions,
  getters,
  mutations,
  state
};

export default userStore;
